'''Fast Avro file iteration.

Most of the code here is ripped off the Python avro package. It's missing a lot
of features in order to get speed.

The only onterface function is iter_avro, example usage::

    import fastavro as avro

    with open('some-file.avro', 'rb') as fo:
        reader = fastavro.reader(fo)
        schema = reader.schema

        for record in reader:
            process_record(record)
'''

__all__ = [ 'iter_avro' ]
__version__ = '0.6.1'


try:
    from . import cfastavro as _avro
except ImportError as e:
    from . import pyfastavro as _avro

reader = iter_avro = _avro.iter_avro




import fastavro as avro
import json
from sys import stdout

encoding = stdout.encoding or "UTF-8"

def main(argv=None):
    import sys
    from argparse import ArgumentParser

    argv = argv or sys.argv

    parser = ArgumentParser(version=avro.__version__,
        description='iter over avro file, emit records as JSON')
    parser.add_argument('file', help='file(s) to parse', nargs='+')
    parser.add_argument('--schema', help='dump schema instead of records',
                        action='store_true', default=False)
    args = parser.parse_args(argv[1:])

    for filename in args.file:
        if filename == '-':
            fo = sys.stdin
        else:
            try:
                fo = open(filename, 'rb')
            except IOError as e:
                raise SystemExit(
                    'error: cannot open {0} - {1}'.format(args.filename, e))

        reader = avro.reader(fo)
        if args.schema:
            json.dump(reader.schema, stdout, indent=4, encoding=encoding)
            sys.stdout.write('\n')
            continue

        try:
            for record in reader:
                json.dump(record, stdout, encoding=encoding)
                sys.stdout.write('\n')
        except (IOError, KeyboardInterrupt):
            pass

if __name__ == '__main__':
    main()




'''Compatiblity for Python versions.

Some of this code is "lifted" from CherryPy.
'''
import sys

_encoding = 'UTF-8'

if sys.version_info >= (3, 0):
    from io import BytesIO as MemoryIO
    xrange = range


    def btou(n, encoding=_encoding):
        return n.decode(encoding)


    def utob(n, encoding=_encoding):
        return bytes(n, encoding)

else:  # Python 2x
    from cStringIO import StringIO as MemoryIO
    xrange = xrange


    def btou(n, encoding=_encoding):
        return n


    def utob(n, encoding=_encoding):
        return n




from setuptools import setup
from distutils.core import Extension
import distutils.log as log
from os.path import join
from distutils.command.build_ext import build_ext
from distutils.errors import CCompilerError, DistutilsExecError, \
    DistutilsPlatformError
import fastavro

cmodule = 'cfastavro'
cfile = join('fastavro', '{}.c'.format(cmodule))

ext_errors = (CCompilerError, DistutilsExecError, DistutilsPlatformError,
              IOError)

class maybe_build_ext(build_ext):
    '''This class allows C extension building to fail.'''

    def run(self):
        try:
            build_ext.run(self)
        except DistutilsPlatformError:
            log.info('cannot bulid C extension, will continue without.')

    def build_extension(self, ext):
        try:
            build_ext.build_extension(self, ext)
        except ext_errors:
            log.info('cannot bulid C extension, will continue without.')


setup(
    name='fastavro',
    version=fastavro.__version__,
    description='Fast iteration of AVRO files',
    long_description=open('README.rst').read(),
    author='Miki Tebeka',
    author_email='miki.tebeka@gmail.com',
    license='MIT',
    url='https://bitbucket.org/tebeka/fastavro',
    packages=['fastavro'],
    ext_modules=[Extension('fastavro.{}'.format(cmodule), [cfile])],
    cmdclass={'build_ext': maybe_build_ext},
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'fastavro = fastavro.__main__:main',
        ]
    },
)
